/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: DefaultObservationPreferences.java,v cb24a43dcaf0 2010/04/29 21:32:27 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.observation;

import javax.annotation.Nonnull;
import java.util.HashMap;
import java.util.Map;
import org.openide.util.Lookup;
import org.openide.util.lookup.ServiceProvider;
import it.tidalwave.util.AsException;
import it.tidalwave.util.Parameters;
import it.tidalwave.bluebill.observation.Cardinality;
import it.tidalwave.bluebill.observation.ObservationItem;
import static it.tidalwave.bluebill.taxonomy.Taxon.Taxon;
import it.tidalwave.bluebill.mobile.taxonomy.TaxonomyPreferences;
import static it.tidalwave.bluebill.mobile.observation.BirdGender.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
@ServiceProvider(service=ObservationPreferences.class)
public class DefaultObservationPreferences implements ObservationPreferences
  {
    private @Nonnull TaxonomyPreferences taxonomyPreferences = Lookup.getDefault().lookup(TaxonomyPreferences.class);

    private final Map<BirdGender, String> map = new HashMap<BirdGender, String>();

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public DefaultObservationPreferences()
      {
        map.put(UNKNOWN, "");
        map.put(MALE, "m");
        map.put(FEMALE, "f");
        map.put(IMMATURE, "imm.");
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public CharSequence format (final @Nonnull ObservationItem observationItem, final @Nonnull Extra... extra)
      {
        return taxonomyPreferences.format(observationItem.getObservable().as(Taxon), formatExtra(observationItem, extra));
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public CharSequence formatAsHtml (final @Nonnull ObservationItem observationItem, final @Nonnull Extra... extra)
      {
        return taxonomyPreferences.formatAsHtml(observationItem.getObservable().as(Taxon), formatExtra(observationItem, extra));
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    private String formatExtra (final @Nonnull ObservationItem observationItem, final @Nonnull Extra... extra)
      {
        final StringBuilder buffer = new StringBuilder();
        String leading = " (";
        String trailing = "";

        if (Parameters.find(Extra.class, extra).contains(Extra.CARDINALITY))
          {
            try
              {
                final Cardinality cardinality = observationItem.getCardinality();

                if (!cardinality.equals(Cardinality.UNDEFINED))
                  {
                    buffer.append(leading);
                    buffer.append(cardinality.toString());
                    leading = "";
                    trailing = ")";
                  }
              }
            catch (AsException e)
              {
                // not present, never mind
              }
          }

        if (Parameters.find(Extra.class, extra).contains(Extra.GENDER))
          {
            try
              {
                final BirdGender gender = observationItem.getObservable().as(BirdGender);

                if (!gender.equals(UNKNOWN))
                  {
                    buffer.append(leading);
                    buffer.append(map.get(gender));
                    trailing = ")";
                  }
              }
            catch (AsException e)
              {
                // not present, never mind
              }
          }

        buffer.append(trailing);

        return buffer.toString();
      }
  }
